/* Parallel Hierarchical Grid -- an adaptive finite element library.
 *
 * Copyright (C) 2005-2007 State Key Laboratory of Scientific and
 * Engineering Computing, Chinese Academy of Sciences. */

/* This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA */

/* $Id: mark.h,v 1.3 2007/02/02 03:30:43 zlb Exp $ */

#ifndef PHG_MARK_H

/******************************************************************************
 * DOF *error and DOF *osc must have one value on each element or each face
 * which is the (L^p norm)^p of the error on the element or face
 * (normally p = 2)
 *                                                                 
 * The type of error indicator defined over element 
 * must be DOF_P0                                                 
 * the type of error indicator defined over face                   
 * must be the same as the type of Face Jump                       
 * the type of data oscillation must be DOF_P0 and                 
 * defined over element                                            
 * ***************************************************************************/

/* Strategies */
enum {
     MARK_DEFAULT,		/* use default strategy */
     MARK_NONE,			/* don't mark any elements */
     MARK_ALL,			/* mark all elements (uniformly refine) */
     MARK_MAX,			/* MAX strategy */
     MARK_GERS,			/* GERS strategy */
     MARK_MNS			/* MNS strategy */
};

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************
 *                  
 *                  Marking function
 *
 *
 *   return value is the number of elements marked (-1 if error)
 *   'theta' >= 0.0, 'zeta' >= 0., 'gamma' >= 0.
 *   'p' >= 1.0, 'coarsen_p' >= 1.
 *   'depth' >= 1, int
 *   'caorsen_depth' >= 1, int
 *
 *   'error', 'osc' and 'coarsen_error' should contain elementwise or facewise
 *   errors to the power of 'p' ('coarsen_p' for 'coarsen_error'),
 *   'error' cannot be NULL except for the first call (initialization),
 *   'osc' and 'coarsen_error' may be NULL
 *
 *   Options for -default_strategy: max, gers, mns, all
 *        default: gers
 *   
 *   It's not necessary to specify the types of DOFs,
 *        the subroutine can probe them, but the strategy and
 *        the types of DOFs must match
 *                  
 *****************************************************************/

long int phgMarkElements_(
	int strategy, DOF *error, FLOAT theta, DOF *osc, FLOAT zeta,
		int depth, FLOAT p,
	int coarsen_strategy, DOF *coarsen_error, FLOAT gamma,
		int coarsen_depth, FLOAT coarsen_p);

#define phgMarkRefine(strategy, error, theta, osc, zeta, depth, p) \
    phgMarkElements_(strategy, error, theta, osc, zeta, depth, p, \
		     MARK_NONE, NULL, 1., 1, p)

#ifdef __cplusplus
}
#endif

#define PHG_MARK_H
#endif
