/* Parallel Hierarchical Grid -- an adaptive finite element library.
 *
 * Copyright (C) 2005-2007 State Key Laboratory of Scientific and
 * Engineering Computing, Chinese Academy of Sciences. */

/* This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA */

/* $Id: tetrahedralize.h,v 1.7 2006/11/21 01:22:41 zlb Exp $ */

#ifndef PHG_TETRAHEDRALIZE_H

typedef void (*ADD_TETRA_FUNC)(int v0, int v1, int v2, int v3,
							int bound_type[]);

typedef enum {
    TETRA	= 0,
    PYRAMID	= 1,
    PRISM	= 2,
    BRICK	= 3
} ELEM_TYPE;

/* Note: FACE_INFO[0] = number of nodes, FACE_INFO[1:] = node list.
 *
 * !!! IMPORTANT: the vertices must be listed circularly which is required by
 * 		  update_neighbours()
 */
typedef short EDGE_INFO[2];	/* list of vertices */
typedef short FACE_INFO[5];	/* 0: # of vertices, 1..: list of vertices */

typedef struct {
    const char  *name;		/* name of the element type */
    EDGE_INFO   *edge_info;	/* list of edges */
    FACE_INFO   *face_info;	/* list of faces */
    short       nvert;		/* number of vertices */
    short	nedge;		/* number of edges */
    short       nface;		/* number of faces */
} ELEM_INFO;

extern int phgElemInfoCount;
extern ELEM_INFO phgElemInfo[];

#ifdef __cplusplus
extern "C" {
#endif

void phgPrism2Tetra(int verts[], ADD_TETRA_FUNC func, int bound_type[]);
void phgBrick2Tetra(int verts[], ADD_TETRA_FUNC func, int bound_type[]);
void phgPyramid2Tetra(int verts[], ADD_TETRA_FUNC func, int bound_type[]);

#ifdef __cplusplus
}
#endif

#define PHG_TETRAHEDRALIZE_H
#endif
